package ch.epfl.tpp.sol;

public final class Card implements Comparable<Card> {
    public enum Suit {
        SPADE("piques"), DIAMOND("carreaux"), CLUB("trèfles"), HEART("cœurs");

        private final String frenchName;
        private Suit(String frenchName) {
            this.frenchName = frenchName;
        }
        
        public final String frenchName() {
            return frenchName;
        }
    };
    
    public enum Rank {
        SIX("six"), SEVEN("sept"), EIGHT("huit"), NINE("neuf"), TEN("dix"),
        JACK("valet"), QUEEN("dame"), KING("roi"), ACE("as");
        
        private final String frenchName;
        private Rank(String frenchName) {
            this.frenchName = frenchName;
        }
        
        public final String frenchName() {
            return frenchName;
        }
    }
    
    private final Suit suit;
    private final Rank rank;
    
    public Card(Suit suit, Rank rank) {
        if (suit == null)
            throw new NullPointerException("null suit");
        if (rank == null)
            throw new NullPointerException("null rank");
        
        this.suit = suit;
        this.rank = rank;
    }
    
    public Suit suit() {
        return suit;
    }
    
    public Rank rank() {
        return rank;
    }
    
    public String frenchName() {
        return rank.frenchName +" de "+ suit.frenchName();
    }
    
    @Override
    public boolean equals(Object that) {
        return that instanceof Card
                && suit == ((Card)that).suit
                && rank == ((Card)that).rank;
    }
    
    @Override
    public int hashCode() {
        return (rank.ordinal() << 2) | suit.ordinal();
    }
    
    @Override
    public int compareTo(Card that) {
        int suitC = suit.compareTo(that.suit);
        return suitC != 0 ? suitC : rank.compareTo(that.rank);
    };
    
    @Override
    public String toString() {
        return rank +" OF "+ suit;
    }
}
