package ch.epfl.tpp.sol;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

public final class JassGame {
    public static List<Card> buildDeck() {
        List<Card> deck = new ArrayList<>(36);
        for (Card.Suit s: Card.Suit.values()) {
            for (Card.Rank r: Card.Rank.values()) {
                deck.add(new Card(s, r));
            }
        }
        return deck;
    }
    
    public static void shuffleDeck(List<Card> deck) {
        Collections.shuffle(deck);
    }
    
    public static List<List<Card>> dealDeck(List<Card> deck, int stacksCount) {
        if (deck.size() % stacksCount != 0)
            throw new IllegalArgumentException("cannot deal deck of "+ deck.size() +" in "+ stacksCount +" equally-sized stacks");
        final int stackSize = deck.size() / stacksCount;
        List<List<Card>> stacks = new ArrayList<>();
        Iterator<Card> cardIt = deck.iterator();
        for (int i = 0; i < stacksCount; ++i) {
            List<Card> stack = new ArrayList<>(stackSize);
            for (int j = 0; j < stackSize; ++j)
                stack.add(cardIt.next());
            stacks.add(stack);
        }
        assert !cardIt.hasNext();
        return stacks;
    }
    
    public static int cardPoints(Card c, Card.Suit trump) {
        switch (c.rank()) {
        case SIX:
        case SEVEN:
        case EIGHT:
            return 0;
        case NINE:
            return c.suit() == trump ? 14 : 0;
        case TEN:
            return 10;
        case JACK:
            return c.suit() == trump ? 20 : 2;
        case QUEEN:
            return 3;
        case KING:
            return 4;
        case ACE:
            return 11;
        }
        throw new IllegalArgumentException(); // cannot happen
    }
}
